@@ -24,6 +24,7 @@ GtkWidget *do_css_basics (GtkWidget *do_widget);
 GtkWidget *do_css_multiplebgs (GtkWidget *do_widget);
 GtkWidget *do_css_pixbufs (GtkWidget *do_widget);
 GtkWidget *do_css_shadows (GtkWidget *do_widget);
+GtkWidget *do_cursors (GtkWidget *do_widget);
 GtkWidget *do_dialog (GtkWidget *do_widget);
 GtkWidget *do_drawingarea (GtkWidget *do_widget);
 GtkWidget *do_editable_cells (GtkWidget *do_widget);
@@ -66,6 +67,7 @@ GtkWidget *do_theming_style_classes (GtkWidget *do_widget);
 GtkWidget *do_toolpalette (GtkWidget *do_widget);
 GtkWidget *do_transparent (GtkWidget *do_widget);
 GtkWidget *do_tree_store (GtkWidget *do_widget);
+GtkWidget *do_pagesetup (GtkWidget *do_widget);
 
 Demo child0[] = {
   { "css_accordion", "CSS Accordion", "css_accordion.c", do_css_accordion, NULL },
@@ -111,6 +113,12 @@ Demo child5[] = {
   { NULL } 
 };
 
+Demo child6[] = {
+  { "printing", "Printing", "printing.c", do_printing, NULL },
+  { "pagesetup", "Page Setup", "pagesetup.c", do_pagesetup, NULL },
+  { NULL } 
+};
+
 Demo gtk_demos[] = {
   { "application", "Application Class", "application.c", do_application, NULL }, 
   { "assistant", "Assistant", "assistant.c", do_assistant, NULL }, 
@@ -121,6 +129,7 @@ Demo gtk_demos[] = {
   { "clipboard", "Clipboard", "clipboard.c", do_clipboard, NULL }, 
   { "colorsel", "Color Chooser", "colorsel.c", do_colorsel, NULL }, 
   { "combobox", "Combo Boxes", "combobox.c", do_combobox, NULL }, 
+  { "cursors", "Cursors", "cursors.c", do_cursors, NULL }, 
   { "dialog", "Dialog and Message Boxes", "dialog.c", do_dialog, NULL }, 
   { "drawingarea", "Drawing Area", "drawingarea.c", do_drawingarea, NULL }, 
   { NULL, "Entry", NULL, NULL, child2 }, 
@@ -142,7 +151,7 @@ Demo gtk_demos[] = {
   { "pickers", "Pickers", "pickers.c", do_pickers, NULL }, 
   { "pixbufs", "Pixbufs", "pixbufs.c", do_pixbufs, NULL }, 
   { "popover", "Popovers", "popover.c", do_popover, NULL }, 
-  { "printing", "Printing", "printing.c", do_printing, NULL }, 
+  { NULL, "Printing", NULL, NULL, child6 }, 
   { "revealer", "Revealer", "revealer.c", do_revealer, NULL }, 
   { "rotated_text", "Rotated Text", "rotated_text.c", do_rotated_text, NULL }, 
   { "sizegroup", "Size Groups", "sizegroup.c", do_sizegroup, NULL }, 
