@@ -8,6 +8,11 @@ struct _ExampleAppWindow
   GtkApplicationWindow parent;
 };
 
+struct _ExampleAppWindowClass
+{
+  GtkApplicationWindowClass parent_class;
+};
+
 typedef struct _ExampleAppWindowPrivate ExampleAppWindowPrivate;
 
 struct _ExampleAppWindowPrivate
