@@ -75,7 +75,23 @@ typedef struct _GdkPoint              GdkPoint;
  * Defines the position and size of a rectangle. It is identical to
  * #cairo_rectangle_int_t.
  */
+#ifdef __GI_SCANNER__
+/* The introspection scanner is currently unable to lookup how
+ * cairo_rectangle_int_t is actually defined. This prevents
+ * introspection data for the GdkRectangle type to include fields
+ * descriptions. To workaround this issue, we define it with the same
+ * content as cairo_rectangle_int_t, but only under the introspection
+ * define.
+ */
+struct _GdkRectangle
+{
+    int x, y;
+    int width, height;
+};
+typedef struct _GdkRectangle          GdkRectangle;
+#else
 typedef cairo_rectangle_int_t         GdkRectangle;
+#endif
 
 /**
  * GdkAtom:
@@ -207,6 +223,10 @@ typedef enum
  * reserved values such as %GDK_MODIFIER_RESERVED_13_MASK.  Your code
  * should preserve and ignore them.  You can use %GDK_MODIFIER_MASK to
  * remove all reserved values.
+ *
+ * Also note that the GDK X backend interprets button press events for button
+ * 4-7 as scroll events, so %GDK_BUTTON4_MASK and %GDK_BUTTON5_MASK will never
+ * be set.
  */
 typedef enum
 {
