@@ -66,6 +66,7 @@ struct _GdkX11Display
 
   gboolean have_randr12;
   gboolean have_randr13;
+  gboolean have_randr15;
   gint xrandr_event_base;
 
   /* If the SECURITY extension is in place, whether this client holds
@@ -130,12 +131,6 @@ struct _GdkX11Display
   gint glx_error_base;
   gint glx_event_base;
 
-  /* Translation between X server time and system-local monotonic time */
-  gint64 server_time_query_time;
-  gint64 server_time_offset;
-
-  guint server_time_is_monotonic_time : 1;
-
   guint have_glx : 1;
 
   /* GLX extensions we check */
