@@ -50,6 +50,7 @@ struct _GtkCssNode
 
   guint                  visible :1;            /* node will be skipped when validating or computing styles */
   guint                  invalid :1;            /* node or a child needs to be validated (even if just for animation) */
+  guint                  children_changed :1;   /* the children changed since last validation */
   guint                  needs_propagation :1;  /* children have state changes that need to be propagated to their siblings */
   /* Two invariants hold for this variable:
    * style_is_invalid == TRUE  =>  next_sibling->style_is_invalid == TRUE
@@ -95,14 +96,10 @@ GType                   gtk_css_node_get_type           (void) G_GNUC_CONST;
 
 void                    gtk_css_node_set_parent         (GtkCssNode            *cssnode,
                                                          GtkCssNode            *parent);
-void                    gtk_css_node_insert_after       (GtkCssNode            *parent,
-                                                         GtkCssNode            *cssnode,
+void                    gtk_css_node_set_after          (GtkCssNode            *cssnode,
                                                          GtkCssNode            *previous_sibling);
-void                    gtk_css_node_insert_before      (GtkCssNode            *parent,
-                                                         GtkCssNode            *cssnode,
+void                    gtk_css_node_set_before         (GtkCssNode            *cssnode,
                                                          GtkCssNode            *next_sibling);
-void                    gtk_css_node_reverse_children   (GtkCssNode            *cssnode);
-
 GtkCssNode *            gtk_css_node_get_parent         (GtkCssNode            *cssnode);
 GtkCssNode *            gtk_css_node_get_first_child    (GtkCssNode            *cssnode);
 GtkCssNode *            gtk_css_node_get_last_child     (GtkCssNode            *cssnode);
@@ -134,8 +131,7 @@ void                    gtk_css_node_remove_class       (GtkCssNode            *
                                                          GQuark                 style_class);
 gboolean                gtk_css_node_has_class          (GtkCssNode            *cssnode,
                                                          GQuark                 style_class);
-const GQuark *          gtk_css_node_list_classes       (GtkCssNode            *cssnode,
-                                                         guint                 *n_classes);
+GList *                 gtk_css_node_list_classes       (GtkCssNode            *cssnode);
 void                    gtk_css_node_add_region         (GtkCssNode            *cssnode,
                                                          GQuark                 region,
                                                          GtkRegionFlags         flags);
