@@ -185,15 +185,7 @@ static void
 gtk_scale_notify (GObject    *object,
                   GParamSpec *pspec)
 {
-  if (strcmp (pspec->name, "orientation") == 0)
-    {
-      GtkOrientation orientation;
-
-      orientation = gtk_orientable_get_orientation (GTK_ORIENTABLE (object));
-      gtk_range_set_flippable (GTK_RANGE (object),
-                               orientation == GTK_ORIENTATION_HORIZONTAL);
-    }
-  else if (strcmp (pspec->name, "inverted") == 0)
+  if (strcmp (pspec->name, "inverted") == 0)
     {
       GtkScale *scale = GTK_SCALE (object);
       GtkScaleMark *mark;
@@ -266,7 +258,6 @@ gtk_scale_class_init (GtkScaleClass *class)
   widget_class->get_preferred_width = gtk_scale_get_preferred_width;
   widget_class->get_preferred_height = gtk_scale_get_preferred_height;
 
-  range_class->slider_detail = "Xscale";
   range_class->get_range_border = gtk_scale_get_range_border;
 
   class->get_layout_offsets = gtk_scale_real_get_layout_offsets;
@@ -307,20 +298,18 @@ gtk_scale_class_init (GtkScaleClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_DIGITS,
                                    g_param_spec_int ("digits",
-						     P_("Digits"),
-						     P_("The number of decimal places that are displayed in the value"),
-						     -1,
-						     MAX_DIGITS,
-						     1,
-						     GTK_PARAM_READWRITE));
+                                                     P_("Digits"),
+                                                     P_("The number of decimal places that are displayed in the value"),
+                                                     -1, MAX_DIGITS, 1,
+                                                     GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
   
   g_object_class_install_property (gobject_class,
                                    PROP_DRAW_VALUE,
                                    g_param_spec_boolean ("draw-value",
-							 P_("Draw Value"),
-							 P_("Whether the current value is displayed as a string next to the slider"),
-							 TRUE,
-							 GTK_PARAM_READWRITE));
+                                                         P_("Draw Value"),
+                                                         P_("Whether the current value is displayed as a string next to the slider"),
+                                                         TRUE,
+                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   g_object_class_install_property (gobject_class,
                                    PROP_HAS_ORIGIN,
@@ -328,25 +317,23 @@ gtk_scale_class_init (GtkScaleClass *class)
                                                          P_("Has Origin"),
                                                          P_("Whether the scale has an origin"),
                                                          TRUE,
-                                                         GTK_PARAM_READWRITE));
+                                                         GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   g_object_class_install_property (gobject_class,
                                    PROP_VALUE_POS,
                                    g_param_spec_enum ("value-pos",
-						      P_("Value Position"),
-						      P_("The position in which the current value is displayed"),
-						      GTK_TYPE_POSITION_TYPE,
-						      GTK_POS_TOP,
-						      GTK_PARAM_READWRITE));
+                                                      P_("Value Position"),
+                                                      P_("The position in which the current value is displayed"),
+                                                      GTK_TYPE_POSITION_TYPE,
+                                                      GTK_POS_TOP,
+                                                      GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY));
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("slider-length",
-							     P_("Slider Length"),
-							     P_("Length of scale's slider"),
-							     0,
-							     G_MAXINT,
-							     31,
-							     GTK_PARAM_READABLE));
+                                           g_param_spec_int ("slider-length",
+                                                             P_("Slider Length"),
+                                                             P_("Length of scale's slider"),
+                                                             0, G_MAXINT, 31,
+                                                             GTK_PARAM_READABLE));
 
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("value-spacing",
@@ -499,8 +486,7 @@ gtk_scale_init (GtkScale *scale)
   priv->digits = 1;
   gtk_range_set_round_digits (range, priv->digits);
 
-  gtk_range_set_flippable (range,
-                           gtk_orientable_get_orientation (GTK_ORIENTABLE (range)) == GTK_ORIENTATION_HORIZONTAL);
+  gtk_range_set_flippable (range, TRUE);
 
   context = gtk_widget_get_style_context (GTK_WIDGET (scale));
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_SCALE);
@@ -904,16 +890,16 @@ gtk_scale_get_range_border (GtkRange  *range,
       if (gtk_orientable_get_orientation (GTK_ORIENTABLE (scale)) == GTK_ORIENTATION_HORIZONTAL)
         {
           if (n1 > 0)
-            border->top += h1 + value_spacing + slider_width / 2;
+            border->top += h1 + value_spacing + slider_width / 4;
           if (n2 > 0)
-            border->bottom += h2 + value_spacing + slider_width / 2;
+            border->bottom += h2 + value_spacing + slider_width / 4;
         }
       else
         {
           if (n1 > 0)
-            border->left += w1 + value_spacing + slider_width / 2;
+            border->left += w1 + value_spacing + slider_width / 4;
           if (n2 > 0)
-            border->right += w2 + value_spacing + slider_width / 2;
+            border->right += w2 + value_spacing + slider_width / 4;
         }
     }
 }
@@ -1176,14 +1162,14 @@ gtk_scale_draw (GtkWidget *widget,
               if (mark->position == GTK_POS_TOP)
                 {
                   y1 = range_rect.y;
-                  y2 = y1 - slider_width / 2;
+                  y2 = y1 - slider_width / 4;
                   min_pos = min_pos_before;
                   max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP) - min_sep;
                 }
               else
                 {
                   y1 = range_rect.y + range_rect.height;
-                  y2 = y1 + slider_width / 2;
+                  y2 = y1 + slider_width / 4;
                   min_pos = min_pos_after;
                   max_pos = find_next_pos (widget, m, marks + i, GTK_POS_BOTTOM) - min_sep;
                 }
@@ -1228,14 +1214,14 @@ gtk_scale_draw (GtkWidget *widget,
               if (mark->position == GTK_POS_TOP)
                 {
                   x1 = range_rect.x;
-                  x2 = range_rect.x - slider_width / 2;
+                  x2 = range_rect.x - slider_width / 4;
                   min_pos = min_pos_before;
                   max_pos = find_next_pos (widget, m, marks + i, GTK_POS_TOP) - min_sep;
                 }
               else
                 {
                   x1 = range_rect.x + range_rect.width;
-                  x2 = range_rect.x + range_rect.width + slider_width / 2;
+                  x2 = range_rect.x + range_rect.width + slider_width / 4;
                   min_pos = min_pos_after;
                   max_pos = find_next_pos (widget, m, marks + i, GTK_POS_BOTTOM) - min_sep;
                 }
@@ -1704,87 +1690,72 @@ marks_start_element (GMarkupParseContext *context,
                      gpointer             user_data,
                      GError             **error)
 {
-  MarksSubparserData *parser_data = (MarksSubparserData*)user_data;
-  guint i;
-  gint line_number, char_number;
+  MarksSubparserData *data = (MarksSubparserData*)user_data;
 
   if (strcmp (element_name, "marks") == 0)
-   ;
+    {
+      if (!_gtk_builder_check_parent (data->builder, context, "object", error))
+        return;
+
+      if (!g_markup_collect_attributes (element_name, names, values, error,
+                                        G_MARKUP_COLLECT_INVALID, NULL, NULL,
+                                        G_MARKUP_COLLECT_INVALID))
+        _gtk_builder_prefix_error (data->builder, context, error);
+    }
   else if (strcmp (element_name, "mark") == 0)
     {
+      const gchar *value_str;
       gdouble value = 0;
-      gboolean has_value = FALSE;
+      const gchar *position_str = NULL;
       GtkPositionType position = GTK_POS_BOTTOM;
       const gchar *msg_context = NULL;
       gboolean translatable = FALSE;
       MarkData *mark;
 
-      for (i = 0; names[i]; i++)
+      if (!_gtk_builder_check_parent (data->builder, context, "marks", error))
+        return;
+
+      if (!g_markup_collect_attributes (element_name, names, values, error,
+                                        G_MARKUP_COLLECT_STRING, "value", &value_str,
+                                        G_MARKUP_COLLECT_BOOLEAN|G_MARKUP_COLLECT_OPTIONAL, "translatable", &translatable,
+                                        G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "comments", NULL,
+                                        G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "context", &msg_context,
+                                        G_MARKUP_COLLECT_STRING|G_MARKUP_COLLECT_OPTIONAL, "position", &position_str,
+                                        G_MARKUP_COLLECT_INVALID))
         {
-          if (strcmp (names[i], "translatable") == 0)
-            {
-              if (!_gtk_builder_boolean_from_string (values[i], &translatable, error))
-                return;
-            }
-          else if (strcmp (names[i], "comments") == 0)
-            {
-              /* do nothing, comments are for translators */
-            }
-          else if (strcmp (names[i], "context") == 0)
-            msg_context = values[i];
-          else if (strcmp (names[i], "value") == 0)
-            {
-              GValue gvalue = G_VALUE_INIT;
-
-              if (!gtk_builder_value_from_string_type (parser_data->builder, G_TYPE_DOUBLE, values[i], &gvalue, error))
-                return;
-
-              value = g_value_get_double (&gvalue);
-              has_value = TRUE;
-            }
-          else if (strcmp (names[i], "position") == 0)
-            {
-              GValue gvalue = G_VALUE_INIT;
-
-              if (!gtk_builder_value_from_string_type (parser_data->builder, GTK_TYPE_POSITION_TYPE, values[i], &gvalue, error))
-                return;
-
-              position = g_value_get_enum (&gvalue);
-            }
-          else
-            {
-              g_markup_parse_context_get_position (context,
-                                                   &line_number,
-                                                   &char_number);
-              g_set_error (error,
-                           GTK_BUILDER_ERROR,
-                           GTK_BUILDER_ERROR_INVALID_ATTRIBUTE,
-                           "%s:%d:%d '%s' is not a valid attribute of <%s>",
-                           "<input>",
-                           line_number, char_number, names[i], "mark");
-              return;
-            }
+          _gtk_builder_prefix_error (data->builder, context, error);
+          return;
         }
 
-      if (!has_value)
+      if (value_str != NULL)
         {
-          g_markup_parse_context_get_position (context,
-                                               &line_number,
-                                               &char_number);
-          g_set_error (error,
-                       GTK_BUILDER_ERROR,
-                       GTK_BUILDER_ERROR_MISSING_ATTRIBUTE,
-                       "%s:%d:%d <%s> requires attribute \"%s\"",
-                       "<input>",
-                       line_number, char_number, "mark",
-                       "value");
-          return;
+          GValue gvalue = G_VALUE_INIT;
+
+          if (!gtk_builder_value_from_string_type (data->builder, G_TYPE_DOUBLE, value_str, &gvalue, error))
+            {
+              _gtk_builder_prefix_error (data->builder, context, error);
+              return;
+            }
+
+          value = g_value_get_double (&gvalue);
+        }
+
+      if (position_str != NULL)
+        {
+          GValue gvalue = G_VALUE_INIT;
+
+          if (!gtk_builder_value_from_string_type (data->builder, GTK_TYPE_POSITION_TYPE, position_str, &gvalue, error))
+            {
+              _gtk_builder_prefix_error (data->builder, context, error);
+              return;
+            }
+
+          position = g_value_get_enum (&gvalue);
         }
 
       mark = g_slice_new (MarkData);
       mark->value = value;
-      if (position == GTK_POS_LEFT ||
-          position == GTK_POS_TOP)
+      if (position == GTK_POS_LEFT || position == GTK_POS_TOP)
         mark->position = GTK_POS_TOP;
       else
         mark->position = GTK_POS_BOTTOM;
@@ -1792,20 +1763,13 @@ marks_start_element (GMarkupParseContext *context,
       mark->context = g_strdup (msg_context);
       mark->translatable = translatable;
 
-      parser_data->marks = g_slist_prepend (parser_data->marks, mark);
+      data->marks = g_slist_prepend (data->marks, mark);
     }
   else
     {
-      g_markup_parse_context_get_position (context,
-                                           &line_number,
-                                           &char_number);
-      g_set_error (error,
-                   GTK_BUILDER_ERROR,
-                   GTK_BUILDER_ERROR_MISSING_ATTRIBUTE,
-                   "%s:%d:%d unsupported tag for GtkScale: \"%s\"",
-                   "<input>",
-                   line_number, char_number, element_name);
-      return;
+      _gtk_builder_error_unhandled_tag (data->builder, context,
+                                        "GtkScale", element_name,
+                                        error);
     }
 }
 
@@ -1840,26 +1804,28 @@ gtk_scale_buildable_custom_tag_start (GtkBuildable  *buildable,
                                       GObject       *child,
                                       const gchar   *tagname,
                                       GMarkupParser *parser,
-                                      gpointer      *data)
+                                      gpointer      *parser_data)
 {
-  MarksSubparserData *parser_data;
+  MarksSubparserData *data;
 
   if (child)
     return FALSE;
 
   if (strcmp (tagname, "marks") == 0)
     {
-      parser_data = g_slice_new0 (MarksSubparserData);
-      parser_data->scale = GTK_SCALE (buildable);
-      parser_data->marks = NULL;
+      data = g_slice_new0 (MarksSubparserData);
+      data->scale = GTK_SCALE (buildable);
+      data->builder = builder;
+      data->marks = NULL;
 
       *parser = marks_parser;
-      *data = parser_data;
+      *parser_data = data;
+
       return TRUE;
     }
 
   return parent_buildable_iface->custom_tag_start (buildable, builder, child,
-                                                   tagname, parser, data);
+                                                   tagname, parser, parser_data);
 }
 
 static void
